﻿using System;
using System.Security.Permissions;
using System.Web;
using System.Web.Caching;
using System.Web.Hosting;
using System.Collections.Generic;

namespace Samples.Bleroy {
    public class MyVpp : VirtualPathProvider {
        private Dictionary<string, string> _files =
            new Dictionary<string,string>(StringComparer.InvariantCultureIgnoreCase) {
                {"code.cs", @"using System;
public class Foo {
    public string Bar {
        get {return ""bar"";}
    }
}"},
                {"page.aspx", @"<%@ Page language=""C#"" %>
<%@ Assembly src=""~/vpp/code.cs"" %>
The date is <%= DateTime.Now.ToString() %><br/>
Foo.Bar: <%= (new Foo()).Bar %>
"}
            };

        public Dictionary<string, string> Files {
            get { return _files; }
        }

        private bool IsPathVirtual(string virtualPath) {
            String checkPath = VirtualPathUtility.ToAppRelative(virtualPath);
            return checkPath.StartsWith("~/vpp", StringComparison.InvariantCultureIgnoreCase);
        }

        public override bool FileExists(string virtualPath) {
            if (IsPathVirtual(virtualPath)) {
                var fileName = VirtualPathUtility.GetFileName(virtualPath);
                return Files.ContainsKey(fileName) ||
                       fileName.StartsWith("calc-", StringComparison.InvariantCultureIgnoreCase);
            }
            else {
                return Previous.FileExists(virtualPath);
            }
        }

        public override bool DirectoryExists(string virtualDir) {
            if (IsPathVirtual(virtualDir)) {
                return true; // all subdirectories exist for now. Woohoo!
            }
            else {
                return Previous.DirectoryExists(virtualDir);
            }
        }

        public override VirtualFile GetFile(string virtualPath) {
            if (IsPathVirtual(virtualPath)) {
                if (FileExists(virtualPath)) {
                    return new MyVirtualFile(virtualPath, this);
                }
                else {
                    return null;
                }
            }
            else {
                return Previous.GetFile(virtualPath);
            }
        }

        public override VirtualDirectory GetDirectory(string virtualDir) {
            if (IsPathVirtual(virtualDir))
                return new MyVirtualDirectory(virtualDir, this);
            else
                return Previous.GetDirectory(virtualDir);
        }

        public override CacheDependency GetCacheDependency(
          string virtualPath,
          System.Collections.IEnumerable virtualPathDependencies,
          DateTime utcStart) {

            if (IsPathVirtual(virtualPath)) {
                return null;
            }
            else
                return Previous.GetCacheDependency(virtualPath, virtualPathDependencies, utcStart);
        }
    }
}